#ifndef __MAGISTREAM_H__
#define __MAGISTREAM_H__

#include "magic/object.h"
#include "magic/cstring.h"

class File;
class DumpContext;
class Stream;
class OStream;

class StreamDevice : public Object {
  public:
};

class File : public StreamDevice {
  protected:
	FILE*	mpFile;
	String	mName;
  public:
						File		(FILE* str=stdout) : mpFile (str) {;}
						File		(const String& n) : mpFile(NULL), mName(n) {;}
	FILE*				getfp		() {return mpFile;}
	bool				isopen		() const {return mpFile;}
	//	bool				open		(bool mode=FMRead);
};

class Stream : public Object {
  protected:
	int		mode;		// Tallennusmoodi
	FILE*	stream;
  public:
						Stream		(FILE* strm) {stream=strm; mode=0;}
	void				operator=	(FILE* strm) {stream=strm; mode=0;}

	void				close		();
	
	friend DumpContext;
};



///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                    ___   ----                                             //
//                   |   | (      |       ___   ___                          //
//                   |   |  ---  -+- |/\ /   )  ___| |/|/|                   //
//                   |   |     )  |  |   |---  (   | | | |                   //
//                   `___ ___/    \ |    \__   \__| | | |                   //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

class OStream : public Stream {
  protected:
	bool			aflush;
	unsigned long	mUserFlags;
  public:
						OStream		(FILE* strm=stdout);
						OStream		(OStream& o);

	OStream&			print		(const char c);
	OStream&			print		(const char* str);
	void				flush		() {fflush (stream);}
	void				autoflush	(bool afl=true) {aflush=afl;}
	void				operator=	(const OStream& other);

	/**
	 * Set user-interpretable formatting flag {0...31}
	 **/
	void				setFlag		(int flag, bool=true);

	/**
	 * Get the value of user flag {0...31}
	 **/
	bool				flag		(int flag) const;

	/**
	 * Sugar for retrieving the user flag
	 **/
	bool				operator[]	(int f) const {return flag(f);}
	
	// Virtual methods
	
	virtual OStream&		printf		(const char* format, ...);
	virtual OStream&		operator<<	(const char* str);
	virtual OStream&		operator<<	(int i);
	virtual OStream&		operator<<	(long i);

};

// Standard output and atandard error output
extern OStream sout;
extern OStream serr;

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//         ___                     ___                                       //
//         |  \               --  /   \        _    |   ___       |          //
//         |   | |   | |/|/| |  ) |      __  |/ \  -+- /   ) \ / -+-         //
//         |   | |   | | | | |--  |     /  \ |   |  |  |---   X   |          //
//         |__/   \__! | | | |    \___/ \__/ |   |   \  \__  / \   \         //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

class DumpContext : public OStream {
  protected:
	String			nextname;	// Seuraavan objektin nimi
	int				depth;		// Sisennystaso
	int				prevdepth;	// Sisennystaso viime tulostuksessa
	Array<String>*	attribs;	// Muistiinpannut attribuutit
	int				errst;
	
	int					open		(const char* filename, int flag);
  public:
	bool	names;
	bool	classnames;
	
						DumpContext		(FILE* strm=stdout);
						DumpContext		(OStream& o);
						~DumpContext	();

	DumpContext&		printf		(const char* format, ...);

	// Implementations

	DumpContext&		operator<<	(int i);
	DumpContext&		operator<<	(long i);
	DumpContext&		operator<<	(char i);
	DumpContext&		operator<<	(float i);
	DumpContext&		operator<<	(double i);
	DumpContext&		operator<<	(const char* str);
	DumpContext&		write		(const char* p, int n);

	// Asettaa seuraavan objektin nimen
	DumpContext&		name		(const char* nam) {nextname=nam; return *this;}

	const Array<String>&	getattribs	() const {return *attribs;}

  protected:
	// Tulostaa nimen (tekstimoodissa)
	DumpContext&		printName	();

	// Tulostaa sisennyksen, jos haluttu
	void				printIndent	();

	// Tulostaa pilkun jos syvyystaso on sama kuin viime kutsulla
	void				printComma	();
	
	friend DumpContext& operator<< (DumpContext& out, const Object& obj);
	friend DumpContext& operator>> (DumpContext& in, Object& obj);
};

// DumpContext& operator<< (DumpContext& dc, int i) {;}

// Sokerointioperaatio name()-liitteen tekemiseksi luokan attribuutteihin
// DumpContext& operator<< (DumpContext& out, const char* str);

enum {	DUMP_LOAD=0, DUMP_SAVE=1, DUMP_RECORD=2,
		DUMP_BINARY=0, DUMP_TEXT=16, DUMP_FORMATTED=32,
		DUMP_VERINFO=64,
		DUMP_ADDRESSES=128, DUMP_CLASSNAMES};



///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                    ---  ----                                              //
//                     |  (      |       ___   ___                           //
//                     |   ---  -+- |/\ /   )  ___| |/|/|                    //
//                     |      )  |  |   |---  (   | | | |                    //
//                    _|_ ___/    \ |    \__   \__| | | |                    //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

class IStream : public Stream {

  public:

				IStream		(FILE* strm=stdin) : Stream (strm) {;}
	
	// Avaa streamin tiedostoon lukua varten, palauttaa FALSE, jos
	// virhetilanne
	bool		openload	(const char* filename = NULL);

	// Lukee kokonaisluvun (todellisuudessa pitkn)
	IStream&	operator>>	(int& i);

	// Lukee pitkn kokonaisluvun
	IStream&	operator>>	(long& i);

	// Lukee yksittisen merkin
	IStream&	operator>>	(char& i);
	IStream&	operator>>	(uchar& i) {return operator>> ((char&)i);}

	// Lukee liukuluvun (todellisuudessa pitkn)
	IStream&	operator>>	(float& i);

	// Lukee pitkn liukuluvun
	IStream&	operator>>	(double& i);
	//	IStream&	operator>>	(char* p)		{if (in) in->read (p, n); return *this;}

	// Lukee muistipuskurin
	IStream&	read		(char* p, int n);

	// const Array<String>&	getattribs	() const {return *attribs;}
};

#endif
