#ifndef __APPLIC_H__
#define __APPLIC_H__

#include "magic/object.h"
#include "magic/Class.h"
#include "magic/map.h"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//               _             | o                 o                        //
//              / \   --   --  |    ___   ___   |           _               //
//             /   \ |  ) |  ) | | |   \  ___| -+- |  __  |/ \              //
//             |---| |--  |--  | | |     (   |  |  | /  \ |   |             //
//             |   | |    |    | |  \__/  \__|   \ | \__/ |   |             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int main (int argc, char** argv);

/** An application-class that can be inherited instead of using main().
 *
 *  If you inherit this, remember to link the app library with switch
 *  -lapp. The application framework automatically calls the main()
 *  method of this class.
 *
 *  NOTE: The applicatino MUST use the decl_dynamic macro to enable house-RTTI.
 **/
class Application : public Object {
	decl_dynamic (Application);
  protected:
	Array<String>	mParams;
	StringMap		mParamMap;
	int				mArgc;
	char**			mArgv;
	int				mTried;

  public:
						Application	() : mTried (0) {}

	/** Always implement this OR the other main () below. OBSOLETE.
	 *
	 *  @param params Parameters passed from command line. This method
	 *  is obsolete because these parameters are now a member.
	 **/
	virtual void		main		(const Array<String>& params) {mTried++;}

	/** Always implement this OR the other main (const Array<String>&) above.
	 **/
	virtual void		main		() {mTried++;}

	/** Reads a configuration file on top of the parameter map.
	 **/
	void				readConfig	(const String& filename);

	/** Returns a reference to the parameter map of the application.
	 **/
	const StringMap&	paramMap	() const {return mParamMap;}

	/** Returns a reference to the parameter map of the application.
	 **/
	const Array<String>& params		() const {return mParams;}
	
	void				setParams	(int argc, char** argv);

  private:
	int					getTried	() const {return mTried;}
};

/**
 *  @short Main() {...} macro to replace main()
 *
 *  @see Application
 **/
#define Main()\
class MainClass : public Application {\
	decl_dynamic (MainClass);\
  public:\
	void main ();\
};\
impl_dynamic (MainClass, "{Application}");\
void MainClass::main ()

extern CClass* appclass;

#endif
