///////////////////////////////////////////////////////////////////////////////
//  Copyright Marko Grnroos 1996, all rights reserved.                     //
// Omistusoikeus Marko Grnroos, 1996, kaikki oikeudet pidtetn.           //
//   Tmn tiedoston kopiointi, lukeminen ja kytt ja erityisesti tmn     //
// ilmoitustekstin muuttaminen on kielletty ilman asianmukaista lupaa.      //
//                                                                           //
//                                             Marko Grnroos (magi@utu.fi)  //
///////////////////////////////////////////////////////////////////////////////

#include "magic/html.h"
#include "magic/Class.h"

decl_module (html);

impl_dynamic (HTMLQuery, {Object});
impl_dynamic (HTMLPageBase, {Object});

///////////////////////////////////////////////////////////////////////////////

// unquote

void unquote (Map<String, String>& map) {
	for (MapIter<String,String> i (map); !i.exhausted(); i.next())
		i.value ().unquote ('%', QUOTE_HTML);
}

///////////////////////////////////////////////////////////////////////////////
//									HTMLQuery

HTMLQuery::HTMLQuery () {
	String contlen_tmp;
	contype		= getenv ("CONTENT_TYPE");
	contlen_tmp	= getenv ("CONTENT_LENGTH");
	docroot		= getenv ("DOCUMENT_ROOT");
	gwif		= getenv ("GATEWAY_INTERFACE");
	hosttype	= getenv ("HOSTTYPE");
	httpaccpt	= getenv ("HTTP_ACCEPT");
	httppragma	= getenv ("HTTP_PRAGMA");
	useragent	= getenv ("HTTP_USER_AGENT");
	pathinfo	= getenv ("PATH_INFO");
	pathtrans	= getenv ("PATH_TRANSLATED");
	querystr	= getenv ("QUERY_STRING");
	remaddr		= getenv ("REMOTE_ADDR");
	remhost		= getenv ("REMOTE_HOST");
	method		= getenv ("REQUEST_METHOD");
	sptname		= getenv ("SCRIPT_NAME");
	srvname		= getenv ("SERVER_NAME");
	srvport		= 80; // atoi (getenv ("SERVER_PORT"));
	srvprot		= getenv ("SERVER_PROTOCOL");
	srvsoft		= getenv ("SERVER_SOFTWARE");

	contlen = contlen_tmp.isnull()? 0 : atoi (contlen_tmp);
//	printf ("contlen=%d\n", (char*) contlen);

	if (method=="GET")
		splitpairs (query, querystr, '=', '&');
	else { // POST
		if (contlen > 0) {
			// Luetaan stdio:ta pituuden verran
			char* buff = new char [contlen+1];
			fgets (buff, contlen+1, stdin);
			querystr = buff;
			delete buff;
			splitpairs (query, querystr, '=', '&');
		}
	}

	// Konvertoidaan query-parametrien quotet
	unquote (query);

//	query.print (cout);
}

void HTMLQuery::printhidden () const {
	forStringMap (query, i)
		printf ("<INPUT TYPE=HIDDEN NAME='%s' VALUE='%s'>\n",
				(CONSTR) i.key (), (CONSTR) i.value ());
}

void HTMLQuery::any () {
	if (pathinfo.len>5 && pathinfo.substr(pathinfo.len-5) == ".html") {
		call (pathinfo.substr (1, pathinfo.len-6)+String("_html"));
	} else
		printf ("Content-type: text/html\n\n"
				"<H1>Invalid path '%s'</H1>\n"
				"Page couldn't be found. Sorry!\n",
				(CONSTR) pathinfo);
	
}

int HTMLQuery::call (const String& pagename) {
	HTMLPageBase* pageobj = (HTMLPageBase*) dyncreate (pagename);
	if (pageobj) {
		pageobj->printqry (*this);
		delete pageobj;
		return 0;
	} else {
		printf ("Content-type: text/html\n\n"
				"<H1>Invalid page called '%s'</H1>\n"
				"Page couldn't be found. Sorry!\n",
				(CONSTR) pathinfo);
		return 1;
	}
}

