///////////////////////////////////////////////////////////////////////////////
//  Copyright Marko Grnroos 1996, all rights reserved.                     //
// Omistusoikeus Marko Grnroos, 1996, kaikki oikeudet pidtetn.           //
//   Tmn tiedoston kopiointi, lukeminen ja kytt ja erityisesti tmn     //
// ilmoitustekstin muuttaminen on kielletty ilman asianmukaista lupaa.      //
//                                                                           //
//                                             Marko Grnroos (magi@utu.fi)  //
///////////////////////////////////////////////////////////////////////////////

//-  Copyright 1992-1995 Marko Grnroos (magi@utu.fi), All rights reserved<P>
//- Standard object hierarchy<BR>
//- Kytetty seuraavissa kyttympristiss, vaikkakaan yhteensopivuus
//- ei vlttmtt ole tysin ajan tasalla:
//-		 MSDOS Borland C++ 3.0
//-		 Unix gcc 2.7.0 C++ (Solaris, Linux)
//-		 Unix egcs C++ (Solaris, Linux)

#ifndef OBJECT_H
#define OBJECT_H 1

//#include "magic/config.h"
#include <typeinfo>	// Used bu dynamic_cast
#include <iostream.h>
#include <stdio.h>
#include "magic/dynamic.h"
#include "magic/debug.h"


//#define stdfloat float
//#define float double
#define CClass Class
#define Object Object
#define String String
#define CArchive DumpContext

class Object;
	class Comparable;

// External classes:
template <class TYPE> class Array;
class Class;
class DumpContext;
class OStream;
class IStream;
class String;


/////////////////////////////////////////////////////////////////////////////

typedef unsigned int	uint;
typedef unsigned char	uchar;

/** Returns 1 (true) if the given object reference is NULL, 0 if not.
 *  Yes, it's not strictly "good C++" to handle references like they
 *  were pointers. But, useful nevertheless.
 **/
inline int isnull (const Object& obj) {return ! (&obj);}

#define isnullref(ref) ((&ref)==NULL)

/** Outputs any @ref Object to the standard C++ stream, if the operator<< has been
 *  defined for it.
 *
 *  For example:
 *
 *  String myString = "Hello, World!";
 *
 *  cout << myString << "\n";
 **/
ostream& operator<< (ostream& out, const Object& obj);

/** Outputs any @ref Object to the MagiClib stream, if the operator<<
 *  has been defined for it.
 *
 *  For example:
 *
 *  String myString = "Hello, World!";
 *
 *  sout << myString << "\n";
 **/
OStream& operator<< (OStream& out, const Object& obj);

/** Reads any @ref Object from the standard C++ stream, if the
 *  operator>> has been defined for it.
 **/
istream& operator>> (istream& in, Object& obj);

/** Outputs any @ref Object to the MagiClib archive, if the operator<<
 *  has been defined for it.
 *
 *  NOTE: The arciving operators have been working fine in past, but
 *  have not been tested for a long time now. They might not work.
 **/
CArchive& operator<< (CArchive& out, const Object& obj);


/** Reads any @ref Object from a MagiClib archive, if the operator>>
 *  has been defined for it.
 *
 *  NOTE: The arciving operators have been working fine in past, but
 *  have not been tested for a long time now. They might not work.
 **/
IStream& operator>> (IStream& in, Object& obj);

///////////////////////////////////////////////////////////////////////////////
//                       ___          o                                      //
//                      |   | |          ___   ___   |                       //
//                      |   | |---    | /   ) |   \ -+-                      //
//                      |   | |   )   | |---  |      |                       //
//                      `___ |__/  \_|  \__   \__/   \                      //
///////////////////////////////////////////////////////////////////////////////

/** Highest abstract baseclass.
 *
 *  NOTE: Not actually abstract because of the RTTI system we use.
 **/
class Object {
	decl_dynamic (Object)
  public:
							Object			()			{}//refs=0;}
	virtual					~Object			();

	/** Serializes the object into a dump stream. You can use this to
	 *  save objects into file.
	 **/
	virtual	DumpContext&	operator>>		(DumpContext&) const;

	/** Ordinary free-formatted printout.
	 **/
	virtual	void			operator>>		(OStream&) const;
	
	/** Loads the object from an archive.
	 *  Overload this to make the class readable.
	 **/
	virtual	IStream&		operator<<		(IStream&);

	/** Printing to standard C++ streams. */
	virtual	ostream&		operator>>		(ostream&) const;
	
	/** Input from standard C++ streams. */
	virtual	istream&		operator<<		(istream&);

	/** Clone should perform a full recursive cloning of the object.
	 *  The return type should vary by implementor.
	 **/
	virtual	Object*			clone			() const;

	/** Checks the integrity of the object, throws check_failed
	 *  exception if fails.
	 **/
	virtual void			check			() const {}

	/** Sugar for the
	 *  @ref check method. Catches the exception and returns a boolean value.
	 **/
	bool					isOK			() const;

	/** Returns the classname if the class has RTTI information
	 *  defined with the decl_dynamic macro.
	 **/
	const String&			getclassname	() const;

	/** Returns true of the object is of the given class. The method
	 *  does not currently check inheritance, as it should.
	 **/
	int						is_a			(const String& classname) const;

#ifdef DEBUG_OBJECT_NEW	
#ifdef new
#undef new
	/** New operator that provides all kinds of nice
	 *  information. Defined only if DEBUG_OBJECT_NEW has been defined
	 *  in debug.h. The new operator is macro-redefined to use this.
	 *
	 *  @see debug.h
	 *
	 *  @param size Size of the allocated block. This is added by the C++ allocation system.
	 *  @param filen File name of the allocation request.
	 *  @param lineno Line number where the allocation was made.
	 *  @param funcn Function name where the allocation was made.
	 **/
	void*					operator new[]	(size_t size, const char* filen,
											 int lineno, const char* funcn);

	/** New operator that provides all kinds of nice
	 *  information. Defined only if DEBUG_OBJECT_NEW has been defined
	 *  in debug.h. The new operator is macro-redefined  to use this.
	 *
	 *  @see debug.h
	 *
	 *  @param size Size of the allocated block. This is added by the C++ allocation system.
	 *  @param filen File name of the allocation request.
	 *  @param lineno Line number where the allocation was made.
	 *  @param funcn Function name where the allocation was made.
	 **/
	void*					operator new	(size_t size, const char* filen,
											 int lineno, const char* funcn);

	/** New operator that provides all kinds of nice
	 *  information. Defined only if DEBUG_OBJECT_NEW has been defined
	 *  in debug.h.
	 **/
	void*					operator new[]	(size_t size);

	/** New operator that provides all kinds of nice
	 *  information. Defined only if DEBUG_OBJECT_NEW has been defined
	 *  in debug.h.
	 **/
	void*					operator new	(size_t size);
#define new DEBUG_NEW
#endif
#endif
	
};



extern const char nullchar;

///////////////////////////////////////////////////////////////////////////////
//             ___                                        |                  //
//            /   \             --   ___       ___  |     |  ___             //
//            |      __  |/|/| |  )  ___| |/\  ___| |---  | /   )            //
//            |     /  \ | | | |--  (   | |   (   | |   ) | |---             //
//            \___/ \__/ | | | |     \__| |    \__| |__/  |  \__             //
///////////////////////////////////////////////////////////////////////////////

/** An @ref Object that can be compared, for sorting purposes, etc. It
 *  has also a hash function for hashing.
 **/
class Comparable : public Object {
	decl_dynamic (Comparable);
  public:
					Comparable	() {;}

	/** Hash function. Inheritors should overload this if they want to
	 *  be used as a key in @ref Map, etc.
	 *
	 *  @return Hash value.
	 *
	 *  @param hashsize Hash size.
	 **/
	virtual int		hashfunc	(int hashsize) const {return 0;}

	/** Equality comparison. Must be overloaded. */
	virtual int		operator==	(const Comparable& other) const=0;

	/** This should be overloaded to enable quicksorting of the
	 *  inheritor.
	 **/
	virtual int		compare		(const Comparable& other) const {return 0;}
};

/** Callback function for running qsort... */
extern int compareComparable	(const void* a, const void* b);

///////////////////////////// Int:Comparable ///////////////////////////

/** Object-integer.
 **/
class Int : public Comparable {
	decl_dynamic (Int)
	long	data;
  public:

				Int				() {data = 0;}
				Int				(const Int& o) {data=o.data;}
				Int				(const String& o);
				Int				(int o) {data=o;}
				Int				(long o) {data=o;}
//				operator int	() const {return data;}
				operator long	() const {return data;}
				operator long&	() {return data;}
//	int			operator <		(const Int& o) {return data<o.data;}
//	int			operator ==		(const Int& o) {return data==o.data;}
	int			hashfunc		(int hashsize) const {return data % hashsize;}
	int			operator ==		(const Comparable& o) const;
//	int			operator !=		(const Int& o) {return data!=o.data;}
	Object*		clone			() const {return new Int (*this);}
};

/** Object-floating poing value.
 **/
class Float : public Comparable {
	double	mValue;
  public:

				Float			() : mValue (0) {}
				Float			(float f) : mValue (f) {}
				Float			(const Float& o) : mValue (o.mValue) {}
	int			operator ==		(const Comparable& o) const {return 0;}
	Object*		clone			() const {return new Float (*this);}
				operator double	() const {return mValue;}
};


///////////////////////////////////////////////////////////////////////////////
//
//								CPPModule
//

/** A module. Some object files may want to register this if they want
 *  to use tracking of how the object files are loaded.
 **/
class CPPModule {
	char* name;
  public:
				CPPModule	(const char* n);
				~CPPModule	() {delete name;}
};

#define decl_module(modname) CPPModule mod_##modname (#modname);


// Ok, include this here now when the Object has been well defined
// #include "magic/Class.h"

///////////////////////////////////////////////////////////////////////////////
// Now we can include these default includes

// Required by magisupp
#include "magic/cstring.h"

// All exception and assertion handling, etc.
#include "magic/magisupp.h"

#endif

//


